@@ -812,14 +812,10 @@ static int process_module_arguments(struct obj_file *f, int argc, char **argv, i
 			memcpy(key + 5, *argv, n);
 			key[n + 5] = '\0';
 			if ((fmt = get_modinfo_value(f, key)) == NULL) {
-				if (required) {
-					error("invalid parameter %s", key);
-					return 0;
-				}
-				else {
-					if (flag_verbose)
-						lprintf("ignoring %s", *argv);
-					continue;	/* silently ignore optional parameters */
+				if (required || flag_verbose) {
+					lprintf("Warning: ignoring %s, no such parameter in this module", *argv);
+					++warnings;
+					continue;
 				}
 			}
 			key += 5;
