@@ -320,7 +320,7 @@ static void update_persistent_data(struct obj_file *f, const char *persist_name,
 {
 	FILE *fr, *fw;
 	int i, fd, len;
-	char *newname = NULL, *newdir = NULL, *p, *name, *value;
+	char *newname = NULL, *newdir = NULL, *p, *name, *value, *uname_m;
 	char line[4096];	/* Grossly oversized for any reasonable text */
 	struct utsname uts;
 	time_t clock;
@@ -363,6 +363,13 @@ static void update_persistent_data(struct obj_file *f, const char *persist_name,
 		error("uname failed: %m");
 		goto err2;
 	}
+	if ((uname_m = getenv("UNAME_MACHINE"))) {
+		int l = strlen(uname_m);
+		if (l >= sizeof(uts.machine))
+			l = sizeof(uts.machine)-1;
+		memcpy(uts.machine, uname_m, l);
+		uts.machine[l] = '\0';
+	}
 
 	fr = fopen(persist_name, "r");
 	fw = fdopen(fd, "w");
