@@ -255,10 +255,12 @@ arch_create_got (struct obj_file *f)
 	    case R_PPC64_ADDR16_HI:
 	    case R_PPC64_ADDR16_LO:
 	    case R_PPC64_ADDR16_LO_DS:
+	    case R_PPC64_ADDR32:
 	    case R_PPC64_ADDR64:
 	    case R_PPC64_TOC:
 	    case R_PPC64_TOC16:
 	    case R_PPC64_TOC16_DS:
+	    case R_PPC64_REL14:
 	      continue;
 
 	    case R_PPC64_REL24:
@@ -374,6 +376,12 @@ arch_finalize_section_address (struct obj_file *f, Elf64_Addr base)
   return 1;
 }
 
+static inline int apply_addr32 (char *loc, Elf64_Addr v)
+{
+  *((Elf64_Word *) loc) = (Elf64_Word) v;
+  return v + ((Elf64_Addr) 1 << 32) >= ((Elf64_Addr) 2 << 32);
+}
+
 static inline int apply_addr16_lo (char *loc, Elf64_Addr v)
 {
   *((Elf64_Half *) loc) = (*((Elf64_Half *) loc) & ~0xffff) | (v & 0xffff);
@@ -448,6 +456,11 @@ arch_apply_relocation (struct obj_file *f,
 	ret = obj_reloc_overflow;
       break;
 
+    case R_PPC64_ADDR32:
+      if (apply_addr32 (loc, v))
+	ret = obj_reloc_overflow;
+      break;
+
     case R_PPC64_ADDR64:
       v = ppc64_adjust_symval (hfile, isym, v);
       *((Elf64_Xword *) loc) = v;
@@ -548,6 +561,20 @@ arch_apply_relocation (struct obj_file *f,
 	      ret = obj_reloc_dangerous;
 	    }
 	}
+      break;
+
+    case R_PPC64_REL14:
+      dot = targsec->header.sh_addr + rel->r_offset;
+      v -= dot;
+      if (v + 0x8000 > 0xffff || (v & 3) != 0)
+	{
+	  if (DEBUG_PPC > 0)
+	    fprintf (stderr, "rel14 overflow\n");
+	  ret = obj_reloc_overflow;
+	}
+      *((Elf64_Word *) loc) = ((*((Elf64_Word *) loc) & ~0xfffc)
+			       | (v & 0xfffc));
+      break;
     }
 
   return ret;
