@@ -493,7 +493,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 	int state[MAX_LEVEL + 1]; /* nested "if" */
 	int level = 0;
 	char buf[3000];
-	char tmpline[100];
+	char tmpline[PATH_MAX];
 	char **pathp;
 	char *envpath;
 	char *version;
